@@ -421,8 +421,9 @@ static void SendVideoInfoFrame(const NVDispEvoRec *pDispEvo,
         head,
         NV_EVO_INFOFRAME_TRANSMIT_CONTROL_EVERY_FRAME,
         (NVT_INFOFRAME_HEADER *) &VideoInfoFrame,
-        /* header length */ sizeof(NVT_INFOFRAME_HEADER) +
-        /* payload length */ VideoInfoFrame.length);
+        (/* header length */ sizeof(NVT_INFOFRAME_HEADER) +
+         /* payload length */ VideoInfoFrame.length),
+        TRUE /* needChecksum */);
 }
 
 /*
@@ -477,8 +478,9 @@ SendHDMI3DVendorSpecificInfoFrame(const NVDispEvoRec *pDispEvo,
         head,
         NV_EVO_INFOFRAME_TRANSMIT_CONTROL_EVERY_FRAME,
         &vendorInfoFrame.Header,
-        /* header length */ sizeof(vendorInfoFrame.Header) +
-        /* payload length */ vendorInfoFrame.Header.length);
+        (/* header length */ sizeof(vendorInfoFrame.Header) +
+         /* payload length */ vendorInfoFrame.Header.length),
+        TRUE /* needChecksum */);
 }
 
 static void
@@ -541,8 +543,9 @@ SendHDRInfoFrame(const NVDispEvoRec *pDispEvo, const NvU32 head,
         head,
         transmitCtrl,
         (NVT_INFOFRAME_HEADER *) &hdrInfoFrame.header,
-        /* header length */ sizeof(hdrInfoFrame.header) +
-        /* payload length */ hdrInfoFrame.header.length);
+        (/* header length */ sizeof(hdrInfoFrame.header) +
+         /* payload length */ hdrInfoFrame.header.length),
+        TRUE /* needChecksum */);
 }
 
 
@@ -911,27 +914,13 @@ static NvBool FillELDBuffer(const NVDpyEvoRec *pDpyEvo,
 
     if (status == NVT_STATUS_SUCCESS) {
         /*
-         * NvTiming_GetProductName() returns a nul-terminated string, but the
-         * string in the EDID is terminated with 0x0A and padded with 0x20.
-         * Put back these special characters.
+         * NvTiming_GetProductName returns a nul-terminated string. Figure out
+         * how long it is and copy the bytes up to, but not including, the nul
+         * terminator.
          */
-        NvBool pastTerminator = FALSE;
-        NvU32 i;
-
-        for (i = 0; i < NVT_EDID_LDD_PAYLOAD_SIZE; i++) {
-            if (pastTerminator) {
-                name[i] = 0x20;
-            }
-            if (name[i] == '\0') {
-                name[i] = 0x0A;
-                pastTerminator = TRUE;
-            }
-        }
-
-        monitorNameLen = NVT_EDID_LDD_PAYLOAD_SIZE;
-        pEld->buffer[4] |= NVT_EDID_LDD_PAYLOAD_SIZE;
-        nvkms_memcpy(&pEld->buffer[20], name,
-                     NVT_EDID_LDD_PAYLOAD_SIZE);
+        monitorNameLen = nvkms_strlen((char *)name);
+        pEld->buffer[4] |= monitorNameLen;
+        nvkms_memcpy(&pEld->buffer[20], name, monitorNameLen);
     }
 
     /* offset 20 + MNL ~ 20 + MNL + (3 * SAD_Count) - 1 : CEA_SADs */
@@ -1280,7 +1269,8 @@ void nvDpyUpdateHdmiVRRCaps(NVDpyEvoPtr pDpyEvo)
 
     const NvBool gpuSupportsHDMIVRR = pDevEvo->hal->caps.supportsHDMIVRR;
 
-    const NvBool dispSupportsVrr = nvDispSupportsVrr(pDispEvo);
+    const NvBool dispSupportsVrr = nvDispSupportsVrr(pDispEvo) && 
+                                       !nvkms_conceal_vrr_caps();
 
     const NvU32 edidVrrMin = pParsedEdid->info.hdmiForumInfo.vrr_min;
 
@@ -1294,8 +1284,6 @@ void nvDpyUpdateHdmiVRRCaps(NVDpyEvoPtr pDpyEvo)
             pDpyEvo->vrr.type =
                 NVKMS_DPY_VRR_TYPE_ADAPTIVE_SYNC_NON_DEFAULTLISTED;
         }
-
-        pDpyEvo->vrr.needsSwFramePacing = TRUE;
     }
 }
 
@@ -1745,7 +1733,8 @@ void nvHdmiSetVRR(NVDispEvoPtr pDispEvo, NvU32 head, NvBool enable)
         head,
         transmitCtrl,
         (NVT_INFOFRAME_HEADER *) &empInfoFrame,
-        sizeof(empInfoFrame));
+        sizeof(empInfoFrame),
+        FALSE /* needChecksum */);
 }
 
 /*
@@ -1899,14 +1888,14 @@ static void HdmiLibPrint(
 }
 
 static void HdmiLibAssert(
-    NvHdmiPkt_CBHandle handle,
-    NvBool expr)
+    const char *expr,
+    const char *filename,
+    const char *function,
+    unsigned int line)
 {
-    /*
-     * This interface isn't the best... I hope you have a kernel debugger if
-     * this fires, because the file and line number will always be this one.
-     */
-    nvAssert(expr);
+#ifdef DEBUG
+    nvDebugAssert(expr, filename, function, line);
+#endif
 }
 
 static NvU64 hdmiLibTimerStartTime = 0;
@@ -2010,7 +1999,6 @@ NvBool nvHdmiFrlAssessLink(NVDpyEvoPtr pDpyEvo)
  */
 NvBool nvHdmiDpySupportsFrl(const NVDpyEvoRec *pDpyEvo)
 {
-    NvU32 passiveDpDongleMaxPclkKHz;
     const NVDevEvoRec *pDevEvo = pDpyEvo->pDispEvo->pDevEvo;
 
     nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
@@ -2025,45 +2013,78 @@ NvBool nvHdmiDpySupportsFrl(const NVDpyEvoRec *pDpyEvo)
         return FALSE;
     }
 
+    /*
+     * Can't use FRL if the connector is not natively HDMI (e.g., if
+     * using a passive DP-to-HDMI dongle, or if overrideEdid/forceConnected
+     * attempted to force HDMI FRL on a DP connector).
+     */
+    if (pDpyEvo->pConnectorEvo->type != NVKMS_CONNECTOR_TYPE_HDMI) {
+        return FALSE;
+    }
+
     /* Can't use FRL if the HDMI sink doesn't support it. */
     if (!pDpyEvo->parsedEdid.valid ||
         !pDpyEvo->parsedEdid.info.hdmiForumInfo.max_FRL_Rate) {
         return FALSE;
     }
 
-    /* Can't use FRL if we are using a passive DP to HDMI dongle. */
-    if (nvDpyGetPassiveDpDongleType(pDpyEvo, &passiveDpDongleMaxPclkKHz) !=
-        NV_EVO_PASSIVE_DP_DONGLE_UNUSED) {
-        return FALSE;
-    }
-
     return TRUE;
 }
 
-NvU32 nvHdmiGetEffectivePixelClockKHz(const NVDpyEvoRec *pDpyEvo,
-                                       const NVHwModeTimingsEvo *pHwTimings,
-                                       const NVDpyAttributeColor *pDpyColor)
+NvBool nvHdmiIsTmdsPossible(const NVDpyEvoRec *pDpyEvo,
+                            const NVHwModeTimingsEvo *pHwTimings,
+                            const NVDpyAttributeColor *pDpyColor)
 {
+    /* For YUV420 HW mode, divide pixel clock by 2. */
     const NvU32 pixelClock = (pHwTimings->yuv420Mode == NV_YUV420_MODE_HW) ?
         (pHwTimings->pixelClock / 2) : pHwTimings->pixelClock;
 
+    ct_assert(NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_BPC_10 ==
+              NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_BPC_MAX);
+
     nvAssert((pHwTimings->yuv420Mode == NV_YUV420_MODE_NONE) ||
                 (pDpyColor->format ==
                  NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr420));
     nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
+
+    /*
+     * HDMI requires 8+ BPC, enforced via nvDpyGetOutputColorFormatInfo() and
+     * IsColorBpcSupported(), so just assert instead of handling < 8 BPC.
+     */
     nvAssert(pDpyColor->bpc >= NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_BPC_8);
 
-    /* YCbCr422 does not change the effective pixel clock. */
-    if (pDpyColor->format ==
-            NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr422) {
-        return pixelClock;
+    /* For YCbCr422, compare without adjusting maximum pixel clock despite BPC. */
+    if (pDpyColor->format == NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr422) {
+        return (pixelClock <= pDpyEvo->maxSingleLinkPixelClockKHz);
     }
 
     /*
-     * For > 8 BPC, the effective pixel clock is adjusted upwards according to
-     * the ratio of the given BPC and 8 BPC.
+     * For 10 BPC, compare with hardware reduced limit if applicable, otherwise
+     * adjust maximum pixel clock by a ratio of 8/10 BPC.
      */
-    return ((pixelClock * pDpyColor->bpc) / 8ULL);
+    if (pDpyColor->bpc == NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_BPC_10) {
+        NvU32 hdmiTmds10BpcMaxPClkKHz =
+            pDpyEvo->pDispEvo->pDevEvo->caps.hdmiTmds10BpcMaxPClkMHz * 1000UL;
+        NvU32 adjustedMaxPixelClock =
+            (pDpyEvo->maxSingleLinkPixelClockKHz * 4ULL) / 5ULL;
+
+        /* Pixel clock must satisfy hdmiTmds10BpcMaxPClkKHz, if applicable. */
+        if ((hdmiTmds10BpcMaxPClkKHz > 0) &&
+            (pixelClock > hdmiTmds10BpcMaxPClkKHz)) {
+            return FALSE;
+        }
+
+        /* Pixel clock must also satisfy adjustedMaxPixelClock. */
+        if (pixelClock > adjustedMaxPixelClock) {
+            return FALSE;
+        }
+
+        return TRUE;
+    }
+
+    /* For 8 BPC, compare without adjusting maximum pixel clock. */
+    nvAssert(pDpyColor->bpc == NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_BPC_8);
+    return (pixelClock <= pDpyEvo->maxSingleLinkPixelClockKHz);
 }
 
 static NvU64 GetHdmiFrlLinkRate(HDMI_FRL_DATA_RATE frlRate)
@@ -2125,8 +2146,7 @@ static NvBool nvHdmiFrlQueryConfigOneBpc(
 
     nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
     nvAssert(nvHdmiDpySupportsFrl(pDpyEvo));
-    nvAssert(nvHdmiGetEffectivePixelClockKHz(pDpyEvo, pHwTimings, pDpyColor) >
-                pDpyEvo->maxSingleLinkPixelClockKHz);
+    nvAssert(!nvHdmiIsTmdsPossible(pDpyEvo, pHwTimings, pDpyColor));
 
     /* See if we can find an NVT_TIMING for this mode from the EDID. */
     pNvtTiming = nvFindEdidNVT_TIMING(pDpyEvo, pModeTimings, pValidationParams);
