@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -32,6 +32,7 @@
 #include "gpu/gsp/kernel_gsp.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/pmu/kern_pmu.h"
+#include "gpu/spdm/spdm.h"
 #include "fsp/nvdm_payload_cmd_response.h"
 #include "fsp/fsp_clock_boost_rpc.h"
 #include "fsp/fsp_caps_query_rpc.h"
@@ -491,7 +492,6 @@ kfspProcessNvdmMessage_GH100
     return status;
 }
 
-
 /*!
  * @brief Process FSP command response
  *
@@ -530,10 +530,11 @@ kfspProcessCommandResponse_GH100
     {
         NV_PRINTF(LEVEL_INFO, "Last command was processed by FSP successfully!\n");
     }
-    else if (status != NV_ERR_OBJECT_NOT_FOUND)
+    else if (status != NV_ERR_OBJECT_NOT_FOUND && status != NV_ERR_INVALID_ARGUMENT)
     {
         NV_PRINTF(LEVEL_ERROR, "FSP response reported error. Task ID: 0x%0x Command type: 0x%0x Error code: 0x%0x\n",
                 pCmdResponse->taskId, pCmdResponse->commandNvdmType, pCmdResponse->errorCode);
+        kfspDumpDebugState_HAL(pGpu, pKernelFsp);
     }
 
     return status;
@@ -801,6 +802,7 @@ kfspWaitForSecureBoot_GH100
 
     if (status != NV_OK)
     {
+        NV_ASSERT_OK(gpuMarkDeviceForReset(pGpu));
         NV_ERROR_LOG((void*) pGpu, GPU_INIT_ERROR, "Error status 0x%x while polling for FSP boot complete, "
                      "0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
                      status,
@@ -830,6 +832,7 @@ kfspGetGspUcodeArchive
 
         if (kgspIsDebugModeEnabled_HAL(pGpu, pKernelGsp))
         {
+
             if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED))
             {
                 return NULL;
@@ -856,6 +859,7 @@ kfspGetGspUcodeArchive
     else
     {
         Gsp *pGsp = GPU_GET_GSP(pGpu);
+        Spdm *pSpdm = GPU_GET_SPDM(pGpu);
 
         // Intentional error print so that we know which mode RM is loaded with
         NV_PRINTF(LEVEL_ERROR, "Loading GSP image for monolithic RM using FSP.\n");
@@ -865,17 +869,6 @@ kfspGetGspUcodeArchive
             {
                 NV_PRINTF(LEVEL_ERROR, "Loading GSP debug inst-in-sys image for monolithic RM using FSP.\n");
 
-                //
-                // GB20X inst-in-sys images will reset PMU, FBFALCON, FECS and GPCCS
-                // and then lower CPUCTL/reset PLM.
-                // Reason for that is that there is fuse configration
-                // changing source isolation of those registers (to GSP/FSP/SEC2).
-                //
-                if (IsGB20X(pGpu))
-                {
-                    pGsp->setProperty(pGsp, PDB_PROP_GSP_INST_IN_SYS_FMC_WILL_RESET_ENGINES, NV_TRUE);
-                }
-
                 return gspGetBinArchiveGspFmcInstInSysGfwDebugSigned_HAL(pGsp);
             }
             else
@@ -889,10 +882,18 @@ kfspGetGspUcodeArchive
                 NV_ASSERT_OR_RETURN(gspSetupRMProxyImage(pGpu, pGsp) == NV_OK, NULL);
 
                 // For debug board, when CC enabled, only pick SPDM profile if SPDM is enabled.
-                if ((pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENABLED)      == NV_TRUE) &&
-                    (pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_SPDM_ENABLED) == NV_TRUE))
+                if  (pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENABLED))
                 {
-                    return gspGetBinArchiveGspFmcSpdmGfwDebugSigned_HAL(pGsp);
+                    if (confComputeIsSpdmEnabled(pGpu, pCC) &&
+                        pSpdm->getProperty(pSpdm, PDB_PROP_SPDM_ENABLED))
+                    {
+                        return gspGetBinArchiveGspFmcSpdmGfwDebugSigned_HAL(pGsp);
+                    }
+                    else
+                    {
+                        // When CC is enabled but SPDM is not enabled. Only for MODS.
+                        return gspGetBinArchiveGspFmcGfwDebugSigned_HAL(pGsp);
+                    }
                 }
                 else
                 {
@@ -906,26 +907,25 @@ kfspGetGspUcodeArchive
             {
                 NV_PRINTF(LEVEL_ERROR, "Loading GSP prod inst-in-sys image for monolithic RM using FSP.\n");
 
-                //
-                // GB20X inst-in-sys images will reset PMU, FBFALCON, FECS and GPCCS
-                // and then lower CPUCTL/reset PLM.
-                // Reason for that is that there is fuse configration
-                // changing source isolation of those registers (to GSP/FSP/SEC2).
-                //
-                if (IsGB20X(pGpu))
-                {
-                    pGsp->setProperty(pGsp, PDB_PROP_GSP_INST_IN_SYS_FMC_WILL_RESET_ENGINES, NV_TRUE);
-                }
-
                 return gspGetBinArchiveGspFmcInstInSysGfwProdSigned_HAL(pGsp);
             }
             else
             {
                 NV_ASSERT_OR_RETURN(gspSetupRMProxyImage(pGpu, pGsp) == NV_OK, NULL);
-                if ((pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENABLED)      == NV_TRUE) &&
-                    (pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_SPDM_ENABLED) == NV_TRUE))
+                Spdm  *pSpdm = GPU_GET_SPDM(pGpu);
+
+                if (pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENABLED))
                 {
-                    return gspGetBinArchiveGspCcFmcGfwProdSigned_HAL(pGsp);
+                    if (confComputeIsSpdmEnabled(pGpu, pCC) &&
+                        pSpdm->getProperty(pSpdm, PDB_PROP_SPDM_ENABLED))
+                    {
+                        return gspGetBinArchiveGspCcFmcGfwProdSigned_HAL(pGsp);
+                    }
+                    else
+                    {
+                        // When CC is enabled but SPDM is not enabled. Only for MODS.
+                        return gspGetBinArchiveGspFmcGfwProdSigned_HAL(pGsp);
+                    }
                 }
                 else
                 {
@@ -952,7 +952,6 @@ kfspGetGspBootArgs
 )
 {
     NV_STATUS status         = NV_OK;
-
     ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
     NV_ASSERT(pCC != NULL);
 
@@ -993,6 +992,7 @@ kfspSetupGspImages
     PBINDATA_STORAGE pGspImageHash;
     PBINDATA_STORAGE pGspImageSignature;
     PBINDATA_STORAGE pGspImagePublicKey;
+    NvBool bReUseInitMem  = pGpu->getProperty(pGpu, PDB_PROP_GPU_REUSE_INIT_CONTING_MEM);
     NvU32 pGspImageSize;
     NvU32 pGspImageMapSize;
     NvP64 pVaKernel = NULL;
@@ -1015,10 +1015,10 @@ kfspSetupGspImages
     }
 
     // Set up the structures to send GSP-FMC
-    pGspImage = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, "ucode_image");
-    pGspImageHash = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, "ucode_hash");
-    pGspImageSignature = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, "ucode_sig");
-    pGspImagePublicKey = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, "ucode_pkey");
+    pGspImage = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, BINDATA_LABEL_UCODE_IMAGE);
+    pGspImageHash = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, BINDATA_LABEL_UCODE_HASH);
+    pGspImageSignature = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, BINDATA_LABEL_UCODE_SIG);
+    pGspImagePublicKey = (PBINDATA_STORAGE)bindataArchiveGetStorage(pBinArchive, BINDATA_LABEL_UCODE_PKEY);
 
     if ((pGspImage == NULL) || (pGspImageHash == NULL) ||
         (pGspImageSignature == NULL) || (pGspImagePublicKey == NULL))
@@ -1029,15 +1029,17 @@ kfspSetupGspImages
 
     pGspImageSize = bindataGetBufferSize(pGspImage);
     pGspImageMapSize = NV_ALIGN_UP(pGspImageSize, 0x1000);
+    if ((pKernelFsp->pGspFmcMemdesc == NULL) || !bReUseInitMem)
+    {
+        NV_ASSERT(pKernelFsp->pGspFmcMemdesc == NULL); // If we assert the pointer becomes a zombie.
+        status = memdescCreate(&pKernelFsp->pGspFmcMemdesc, pGpu, pGspImageMapSize,
+                               0, NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED, flags);
+        NV_ASSERT_OR_GOTO(status == NV_OK, failed);
 
-    status = memdescCreate(&pKernelFsp->pGspFmcMemdesc, pGpu, pGspImageMapSize,
-                           0, NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED, flags);
-    NV_ASSERT_OR_GOTO(status == NV_OK, failed);
-
-    memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_7,
-                    pKernelFsp->pGspFmcMemdesc);
-    NV_ASSERT_OR_GOTO(status == NV_OK, failed);
-
+        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_7,
+                        pKernelFsp->pGspFmcMemdesc);
+        NV_ASSERT_OR_GOTO(status == NV_OK, failed);
+    }
     status = memdescMap(pKernelFsp->pGspFmcMemdesc, 0, pGspImageMapSize, NV_TRUE,
                         NV_PROTECT_READ_WRITE, &pVaKernel, &pPrivKernel);
     NV_ASSERT_OR_GOTO(status == NV_OK, failed);
@@ -1048,7 +1050,7 @@ kfspSetupGspImages
     NV_ASSERT_OR_GOTO(status == NV_OK, failed);
 
     // Clean up CPU side resources since they are not needed anymore
-    memdescUnmap(pKernelFsp->pGspFmcMemdesc, NV_TRUE, 0, pVaKernel, pPrivKernel);
+    memdescUnmap(pKernelFsp->pGspFmcMemdesc, NV_TRUE, pVaKernel, pPrivKernel);
 
     pCotPayload->gspFmcSysmemOffset = memdescGetPhysAddr(pKernelFsp->pGspFmcMemdesc, AT_GPU, 0);
 
@@ -1100,11 +1102,10 @@ _kfspIsGspTargetMaskReleased
     NvU32 reg;
 
     //
-    // This register is read with the raw OS read to avoid the 0xbadf sanity checking
+    // This register is read with GPU_REG_RD32_UNCHECKED to avoid the 0xbadf sanity checking
     // done by the usual register read utilities.
     //
-    reg = osDevReadReg032(pGpu, gpuGetDeviceMapping(pGpu, DEVICE_INDEX_GPU, 0),
-                          DRF_BASE(NV_PGSP) + NV_PFALCON_FALCON_HWCFG2);
+    reg = GPU_REG_RD32_UNCHECKED(pGpu, DRF_BASE(NV_PGSP) + NV_PFALCON_FALCON_HWCFG2);
 
     return ((reg != 0) && ((reg & privErrTargetLockedMask) != privErrTargetLocked));
 }
@@ -1289,9 +1290,8 @@ kfspPrepareBootCommands_GH100
     NvU32 frtsSize = 0;
     NvP64 pVaKernel = NULL;
     NvP64 pPrivKernel = NULL;
-    NvBool bIsKeepWPRGc6D3Cold = pGpu->getProperty(pGpu, PDB_PROP_GPU_KEEP_WPR_ACROSS_GC6_SUPPORTED) &&
-                                 IS_GPU_GC6_STATE_EXITING(pGpu) &&
-                                 pKernelFsp->bUseKeepWPRGc6FSPCommand;
+    NvBool bIsKeepWPRGc6 = IS_GPU_GC6_STATE_EXITING(pGpu);
+    NvBool bReUseInitMem  = pGpu->getProperty(pGpu, PDB_PROP_GPU_REUSE_INIT_CONTING_MEM);
 
     statusBoot = kfspWaitForSecureBoot_HAL(pGpu, pKernelFsp);
 
@@ -1374,13 +1374,17 @@ kfspPrepareBootCommands_GH100
         // FSP (an unit inside GPU) and hence placed in unprotected sysmem
         //
         flags = MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY;
-        status = memdescCreate(&pKernelFsp->pSysmemFrtsMemdesc, pGpu, frtsSize,
-                               0, NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED, flags);
-        NV_ASSERT_OR_GOTO(status == NV_OK, failed);
+        if ((pKernelFsp->pSysmemFrtsMemdesc == NULL) || !bReUseInitMem)
+        {
+            NV_ASSERT(pKernelFsp->pSysmemFrtsMemdesc == NULL); // If we assert the pointer becomes a zombie.
+            status = memdescCreate(&pKernelFsp->pSysmemFrtsMemdesc, pGpu, frtsSize,
+                                   0, NV_TRUE, ADDR_SYSMEM, NV_MEMORY_CACHED, flags);
+            NV_ASSERT_OR_GOTO(status == NV_OK, failed);
 
-        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_8,
-                        pKernelFsp->pSysmemFrtsMemdesc);
-        NV_ASSERT_OR_GOTO(status == NV_OK, failed);
+            memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_8,
+                            pKernelFsp->pSysmemFrtsMemdesc);
+            NV_ASSERT_OR_GOTO(status == NV_OK, failed);
+        }
 
         // Set up a kernel mapping for future use in RM
         status = memdescMap(pKernelFsp->pSysmemFrtsMemdesc, 0, frtsSize, NV_TRUE,
@@ -1404,7 +1408,7 @@ kfspPrepareBootCommands_GH100
     // Set up vidmem for FRTS copy
     // With Keep WPR feature, set Frts_In_Fb_Requested to False for VBIOS.
     //
-    if (!pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_FRTS_VIDMEM) && !bIsKeepWPRGc6D3Cold)
+    if (!pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_FRTS_VIDMEM) && !bIsKeepWPRGc6)
     {
         //
         // Since we are very early in the boot path, we cannot know how much
@@ -1458,7 +1462,7 @@ kfspPrepareBootCommands_GH100
         pKernelFsp->pCotPayload->frtsVidmemSize = 0;
     }
 
-    if (!bIsKeepWPRGc6D3Cold)
+    if (!bIsKeepWPRGc6)
     {
         pKernelFsp->pCotPayload->gspFmcSysmemOffset = (NvU64)-1;
         pKernelFsp->pCotPayload->gspBootArgsSysmemOffset = (NvU64)-1;
@@ -1468,7 +1472,7 @@ kfspPrepareBootCommands_GH100
     if (!pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_GSPFMC))
     {
         // With Keep WPR feature, we can reuse the GSP-FMC image in FB.
-        if (!bIsKeepWPRGc6D3Cold)
+        if (!bIsKeepWPRGc6)
         {
             status = kfspSetupGspImages(pGpu, pKernelFsp, pKernelFsp->pCotPayload);
             if (status!= NV_OK)
@@ -1574,7 +1578,6 @@ kfspPrepareAndSendBootCommands_GH100
 {
     NV_STATUS status;
     status = kfspPrepareBootCommands_GH100(pGpu, pKernelFsp);
-    pKernelFsp->bUseKeepWPRGc6FSPCommand = NV_FALSE;
     if (status != NV_OK)
     {
         return (status == NV_WARN_NOTHING_TO_DO) ? NV_OK : status;
@@ -1602,6 +1605,9 @@ kfspErrorCode2NvStatusMap_GH100
         case FSP_ERR_IFS_ERR_INVALID_DATA:
         return NV_ERR_INVALID_DATA;
 
+        case FSP_ERR_PRC_ERROR_INVALID_KNOB_ID:
+        return NV_ERR_INVALID_ARGUMENT;
+
         default:
         return NV_ERR_GENERIC;
     }
