@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,6 +34,7 @@
 #include "gpu/gpu.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
 #include "kernel/gpu/nvlink/kernel_ioctrl.h"
+#include "kernel/gpu/nvlink/common_nvlink.h"
 #include "kernel/gpu/device/device.h"
 #include "kernel/gpu/bus/p2p_api.h"
 #include "kernel/gpu/fifo/kernel_channel.h"
@@ -43,6 +44,7 @@
 #include "gpu/ce/kernel_ce.h"
 #include "platform/sli/sli.h"
 #include "gpu/gpu_fabric_probe.h"
+#include "swref/common_def_nvlink.h"
 #include "compute/imex_session_api.h"
 #include "compute/fabric.h"
 #include "mem_mgr/mem_multicast_fabric.h"
@@ -250,8 +252,8 @@ _knvlinkCheckFabricCliqueId
                                              &cliqueId);
     if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "GPU %d failed to get fabric clique Id: 0x%x\n",
-                                gpuGetInstance(pGpu), status);
+        NV_PRINTF(LEVEL_INFO, "GPU %d failed to get fabric clique Id: 0x%x\n",
+                  gpuGetInstance(pGpu), status);
         return NV_FALSE;
     }
 
@@ -259,14 +261,14 @@ _knvlinkCheckFabricCliqueId
                                              &peerCliqueId);
     if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "GPU %d failed to get fabric clique Id 0x%x\n",
-                                gpuGetInstance(pPeerGpu), status);
+        NV_PRINTF(LEVEL_INFO, "GPU %d failed to get fabric clique Id 0x%x\n",
+                  gpuGetInstance(pPeerGpu), status);
         return NV_FALSE;
     }
 
     if (cliqueId != peerCliqueId)
     {
-        NV_PRINTF(LEVEL_ERROR, "GPU %d and Peer GPU %d cliqueId doesn't match\n",
+        NV_PRINTF(LEVEL_INFO, "GPU %d and Peer GPU %d cliqueId doesn't match\n",
                   gpuGetInstance(pGpu), gpuGetInstance(pPeerGpu));
         return NV_FALSE;
     }
@@ -816,10 +818,12 @@ knvlinkInbandMsgCallbackDispatcher_IMPL
     pData->dataSize = dataSize;
     portMemCopy(pData->data, pData->dataSize, pMessage, dataSize);
 
-    status = osQueueWorkItemWithFlags(pGpu, knvlinkInbandMsgCallbackDispatcher_WORKITEM, pData,
-                                      pCb->wqItemFlags);
-     if (status != NV_OK)
-     {
+    status = osQueueWorkItem(pGpu,
+                             knvlinkInbandMsgCallbackDispatcher_WORKITEM,
+                             pData,
+                             pCb->wqItemFlags);
+    if (status != NV_OK)
+    {
         portMemFree(pData);
         return status;
      }
@@ -918,12 +922,12 @@ knvlinkGetNumLinksToPeer_IMPL
     OBJGPU       *pRemoteGpu
 )
 {
-    NvU32 numPeerLinks =
+    NvU64 numPeerLinks =
         knvlinkGetLinkMaskToPeer(pGpu, pKernelNvlink, pRemoteGpu);
 
     if (numPeerLinks != 0)
     {
-        NUMSETBITS_32(numPeerLinks);
+        numPeerLinks = nvPopCount64(numPeerLinks);
     }
 
     return numPeerLinks;
@@ -938,7 +942,7 @@ knvlinkGetNumLinksToPeer_IMPL
  *
  * @return    Returns the mask of peer links between the GPUs
  */
-NvU32
+NvU64
 knvlinkGetLinkMaskToPeer_IMPL
 (
     OBJGPU       *pGpu0,
@@ -946,7 +950,7 @@ knvlinkGetLinkMaskToPeer_IMPL
     OBJGPU       *pGpu1
 )
 {
-    NvU32 peerLinkMask = 0;
+    NvU64 peerLinkMask = 0;
     KernelNvlink *pKernelNvlink1 = NULL;
 
     pKernelNvlink1 = GPU_GET_KERNEL_NVLINK(pGpu1);
@@ -976,7 +980,7 @@ knvlinkGetLinkMaskToPeer_IMPL
         // are updated only when a P2P object is allocated. So, return
         // the cached value of mask of links connected to a GPU
         //
-        peerLinkMask = pKernelNvlink0->peerLinkMasks[gpuGetInstance(pGpu1)];
+        peerLinkMask = KNVLINK_GET_MASK(pKernelNvlink0, peerLinkMasks[gpuGetInstance(pGpu1)], 64);
     }
 
     return peerLinkMask;
@@ -998,13 +1002,13 @@ knvlinkSetLinkMaskToPeer_IMPL
     OBJGPU       *pGpu0,
     KernelNvlink *pKernelNvlink0,
     OBJGPU       *pGpu1,
-    NvU32         peerLinkMask
+    NvU64         peerLinkMask
 )
 {
     NV_STATUS status = NV_OK;
 
     // Return early if no update needed to the peer link mask
-    if (pKernelNvlink0->peerLinkMasks[gpuGetInstance(pGpu1)] == peerLinkMask)
+    if (KNVLINK_GET_MASK(pKernelNvlink0, peerLinkMasks[gpuGetInstance(pGpu1)], 64) == peerLinkMask)
         return NV_OK;
 
     pKernelNvlink0->peerLinkMasks[gpuGetInstance(pGpu1)] = peerLinkMask;
@@ -1036,54 +1040,6 @@ knvlinkSetLinkMaskToPeer_IMPL
     return NV_OK;
 }
 
-/*!
- * @brief Get the mask of links that are peer links
- *
- * @param[in] pGpu           OBJGPU pointer
- * @param[in] pKernelNvlink  KernelNvlink pointer
- */
-NvU32
-knvlinkGetPeersNvlinkMaskFromHshub_IMPL
-(
-    OBJGPU       *pGpu,
-    KernelNvlink *pKernelNvlink
-)
-{
-    NV_STATUS status       = NV_OK;
-    NvU32     peerLinkMask = 0;
-    NvU32     i;
-
-    NV2080_CTRL_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_INFO_PARAMS *pParams;
-
-    pParams = portMemAllocStackOrHeap(sizeof(*pParams));
-    if (pParams == NULL)
-    {
-        return 0;
-    }
-
-    portMemSet(pParams, 0, sizeof(*pParams));
-    pParams->linkMask = KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32);
-    pParams->bSublinkStateInst = NV_TRUE;
-
-    status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_INFO,
-                                 pParams, sizeof(*pParams));
-    if (status != NV_OK)
-        goto cleanup;
-
-    // Scan enabled links for peer connections
-    FOR_EACH_INDEX_IN_MASK(32, i, KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32))
-    {
-        if (pParams->linkInfo[i].bLinkConnectedToPeer)
-            peerLinkMask |= NVBIT(i);
-    }
-    FOR_EACH_INDEX_IN_MASK_END;
-
-cleanup:
-    portMemFreeStackOrHeap(pParams);
-    return peerLinkMask;
-}
-
 /*!
  * @brief Prepare a GPU's NVLink engine for reset by removing mappings
  *        to it from other GPUs.
@@ -1250,7 +1206,7 @@ knvlinkPrepareForXVEReset_IMPL
             //
             FOR_EACH_INDEX_IN_MASK(32, linkId, KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32))
             {
-                pKernelNvlink->disconnectedLinkMask |=  NVBIT(linkId);
+                pKernelNvlink->disconnectedLinkMask |=  NVBIT64(linkId);
                 pKernelNvlink->connectedLinksMask   &= ~NVBIT(linkId);
 
                 if (pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.deviceType !=
@@ -1269,7 +1225,7 @@ knvlinkPrepareForXVEReset_IMPL
                     KernelNvlink *pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
                     NvU32 remoteLinkId = pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber;
 
-                    pRemoteKernelNvlink->disconnectedLinkMask |=  NVBIT(remoteLinkId);
+                    pRemoteKernelNvlink->disconnectedLinkMask |=  NVBIT64(remoteLinkId);
                     pRemoteKernelNvlink->connectedLinksMask   &= ~NVBIT(remoteLinkId);
                 }
             }
@@ -1571,6 +1527,7 @@ knvlinkSyncLinkMasksAndVbiosInfo_IMPL
 )
 {
     NV_STATUS status = NV_OK;
+    NV2080_NVLINK_BIT_VECTOR localLinkMaskBitVector;
 
     NV2080_CTRL_NVLINK_SYNC_LINK_MASKS_AND_VBIOS_INFO_PARAMS params;
 
@@ -1581,6 +1538,15 @@ knvlinkSyncLinkMasksAndVbiosInfo_IMPL
     params.bridgeSensableLinks = pKernelNvlink->bridgeSensableLinks;
     params.bridgedLinks        = pKernelNvlink->bridgedLinks;
 
+    params.discoveredLinkMasks.masks[0] = pKernelNvlink->discoveredLinks;
+    params.discoveredLinkMasks.lenMasks = 1;
+    params.connectedLinks.masks[0] = pKernelNvlink->connectedLinksMask;
+    params.connectedLinks.lenMasks = 1;
+    params.bridgeSensableLinkMasks.masks[0] = pKernelNvlink->bridgeSensableLinks;
+    params.bridgeSensableLinkMasks.lenMasks = 1;
+    params.bridgedLinkMasks.masks[0] = pKernelNvlink->bridgedLinks;
+    params.bridgedLinkMasks.lenMasks = 1;
+
     // Reset timeout to clear any accumulated timeouts from link init
     if (IS_GSP_CLIENT(pGpu))
     {
@@ -1591,9 +1557,20 @@ knvlinkSyncLinkMasksAndVbiosInfo_IMPL
                                  NV2080_CTRL_CMD_NVLINK_SYNC_LINK_MASKS_AND_VBIOS_INFO,
                                  (void *)&params, sizeof(params));
 
-    pKernelNvlink->vbiosDisabledLinkMask = params.vbiosDisabledLinkMask;
-    pKernelNvlink->initializedLinks      = params.initializedLinks;
-    pKernelNvlink->initDisabledLinksMask = params.initDisabledLinksMask;
+    convertLinkMasksToBitVector(&params.vbiosDisabledLinkMask, sizeof(params.vbiosDisabledLinkMask),
+                                &params.vbiosDisabledLinks, &localLinkMaskBitVector);
+    convertBitVectorToLinkMasks(&localLinkMaskBitVector, &pKernelNvlink->vbiosDisabledLinkMask,
+                                sizeof(pKernelNvlink->vbiosDisabledLinkMask), NULL);
+    bitVectorClrAll(&localLinkMaskBitVector);
+    convertLinkMasksToBitVector(&params.initializedLinks, sizeof(params.initializedLinks),
+                                &params.initializedLinkMasks, &localLinkMaskBitVector);
+    convertBitVectorToLinkMasks(&localLinkMaskBitVector, &pKernelNvlink->initializedLinks,
+                                sizeof(pKernelNvlink->initializedLinks), NULL);
+    bitVectorClrAll(&localLinkMaskBitVector);
+    convertLinkMasksToBitVector(&params.initDisabledLinksMask, sizeof(params.initDisabledLinksMask),
+                                &params.initDisabledLinks, &localLinkMaskBitVector);
+    convertBitVectorToLinkMasks(&localLinkMaskBitVector, &pKernelNvlink->initDisabledLinksMask,
+                                sizeof(pKernelNvlink->initDisabledLinksMask), NULL);
     pKernelNvlink->bEnableSafeModeAtLoad = params.bEnableSafeModeAtLoad;
     pKernelNvlink->bEnableTrainingAtLoad = params.bEnableTrainingAtLoad;
 
@@ -1828,13 +1805,14 @@ knvlinkCopyNvlinkDeviceInfo_IMPL
     }
 
     // Update CPU-RM's NVLink state with the information received from GSP-RM RPC
-    pKernelNvlink->ioctrlMask       = pNvlinkInfoParams->ioctrlMask;
-    pKernelNvlink->ioctrlNumEntries = pNvlinkInfoParams->ioctrlNumEntries;
-    pKernelNvlink->ioctrlSize       = pNvlinkInfoParams->ioctrlSize;
-    pKernelNvlink->discoveredLinks  = pNvlinkInfoParams->discoveredLinks;
-    pKernelNvlink->ipVerNvlink      = pNvlinkInfoParams->ipVerNvlink;
+    pKernelNvlink->ioctrlMask        = pNvlinkInfoParams->ioctrlMask;
+    pKernelNvlink->ioctrlNumEntries  = pNvlinkInfoParams->ioctrlNumEntries;
+    pKernelNvlink->ioctrlSize        = pNvlinkInfoParams->ioctrlSize;
+    pKernelNvlink->discoveredLinks   = pNvlinkInfoParams->discoveredLinks;
+    pKernelNvlink->ipVerNvlink       = pNvlinkInfoParams->ipVerNvlink;
+    pKernelNvlink->maxSupportedLinks = pNvlinkInfoParams->maxSupportedLinks;
 
-    for (i = 0; i < NVLINK_MAX_LINKS_SW; i++)
+    for (i = 0; i < pKernelNvlink->maxSupportedLinks; i++)
     {
         pKernelNvlink->nvlinkLinks[i].pGpu     = pGpu;
         pKernelNvlink->nvlinkLinks[i].bValid   = pNvlinkInfoParams->linkInfo[i].bValid;
@@ -2116,13 +2094,14 @@ knvlinkProcessInitDisabledLinks_IMPL
     KernelNvlink *pKernelNvlink
 )
 {
-    NvU32     mask                 = 0;
+    NV2080_CTRL_NVLINK_LINK_MASK links = {0};
     NvBool    bSkipHwNvlinkDisable = 0;
     NV_STATUS status               = NV_OK;
+    NV2080_NVLINK_BIT_VECTOR localLinkMask;
 
-    NV2080_CTRL_NVLINK_PROCESS_INIT_DISABLED_LINKS_PARAMS params;
+    NV2080_CTRL_NVLINK_PROCESS_INIT_DISABLED_LINKS_PARAMS params = {0};
 
-    status = gpumgrGetGpuInitDisabledNvlinks(pGpu->gpuId, &mask, &bSkipHwNvlinkDisable);
+    status = gpumgrGetGpuInitDisabledNvlinks(pGpu->gpuId, &links, &bSkipHwNvlinkDisable);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to get init disabled links from gpumgr\n");
@@ -2131,8 +2110,8 @@ knvlinkProcessInitDisabledLinks_IMPL
 
     portMemSet(&params, 0, sizeof(params));
 
-    params.initDisabledLinksMask = mask;
-    params.bSkipHwNvlinkDisable  = bSkipHwNvlinkDisable;
+    params.initDisabledLinks = links;
+    params.bSkipHwNvlinkDisable = bSkipHwNvlinkDisable;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
                                  NV2080_CTRL_CMD_NVLINK_PROCESS_INIT_DISABLED_LINKS,
@@ -2143,7 +2122,12 @@ knvlinkProcessInitDisabledLinks_IMPL
         return status;
     }
 
-    pKernelNvlink->initDisabledLinksMask = params.initDisabledLinksMask;
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        convertLinkMasksToBitVector(NULL, 0, &params.initDisabledLinks, &localLinkMask));
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        convertBitVectorToLinkMasks(&localLinkMask, &pKernelNvlink->initDisabledLinksMask,
+                                    sizeof(pKernelNvlink->initDisabledLinksMask), NULL));
 
     return NV_OK;
 }
@@ -2420,8 +2404,10 @@ knvlinkUncontainedErrorRecovery_WORKITEM
 
     // Launch lockless workitem to resume P2P in UVM
     NV_ASSERT_OK_OR_GOTO(status,
-        osQueueWorkItemWithFlags(pGpu, knvlinkUncontainedErrorRecoveryUvmResume_WORKITEM, pInfo,
-                                 (OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
+        osQueueWorkItem(pGpu,
+                        knvlinkUncontainedErrorRecoveryUvmResume_WORKITEM,
+                        pInfo,
+                        (OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
         exit);
 
 exit:
@@ -2437,12 +2423,12 @@ exit:
     if (bDegrade)
     {
         NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
-            osQueueWorkItemWithFlags(pGpu, knvlinkAbortUncontainedErrorRecovery_WORKITEM, NULL,
-                                     (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)));
-
-
+            osQueueWorkItem(pGpu,
+                knvlinkAbortUncontainedErrorRecovery_WORKITEM,
+                NULL,
+                (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
+                 OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
+                 OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)));
     }
 }
 
@@ -2516,18 +2502,24 @@ knvlinkFatalErrorRecovery_IMPL
         }
 
         // Launch workitem to RC outstanding IMEX clients
-        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-            osQueueWorkItemWithFlags(pGpu, knvlinkFatalErrorRecovery_WORKITEM, pInfo,
-                                     (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE |
-                                       OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
+        NV_CHECK_OK_OR_GOTO(status,
+            LEVEL_ERROR,
+            osQueueWorkItem(pGpu,
+                            knvlinkFatalErrorRecovery_WORKITEM,
+                            pInfo,
+                            (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
+                             OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
+                             OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE |
+                             OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
             fail);
 
         // Launch lockless workitem to idle UVM channels
-        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-            osQueueWorkItemWithFlags(pGpu, knvlinkUncontainedErrorRecoveryUvmIdle_WORKITEM, pInfo,
-                                     (OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
+        NV_CHECK_OK_OR_GOTO(status,
+            LEVEL_ERROR,
+            osQueueWorkItem(pGpu,
+                            knvlinkUncontainedErrorRecoveryUvmIdle_WORKITEM,
+                            pInfo,
+                            (OS_QUEUE_WORKITEM_FLAGS_DONT_FREE_PARAMS)),
             fail);
 
         // Launch repeated 1Hz workitem to wait 1 STO period
@@ -2544,20 +2536,24 @@ knvlinkFatalErrorRecovery_IMPL
     {
         (void)gpuMarkDeviceForReset(pGpu);
 
-        status = osQueueWorkItemWithFlags(pGpu, knvlinkFatalErrorRecovery_WORKITEM, NULL,
-                                          (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                            OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
-                                            OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE));
+        status = osQueueWorkItem(pGpu,
+            knvlinkFatalErrorRecovery_WORKITEM,
+            NULL,
+            (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
+             OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
+             OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE));
     }
 
     return status;
 
 fail:
     NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
-        osQueueWorkItemWithFlags(pGpu, knvlinkAbortUncontainedErrorRecovery_WORKITEM, NULL,
-                                 (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                   OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
-                                   OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)));
+        osQueueWorkItem(pGpu,
+                        knvlinkAbortUncontainedErrorRecovery_WORKITEM,
+                        NULL,
+                        (OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
+                         OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RW |
+                         OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)));
 
     return status;
 }
@@ -2610,6 +2606,27 @@ knvlinkExecGspRmRpc_IMPL
     return status;
 }
 
+NvU64
+knvlinkGetBWModeEpoch_IMPL
+(
+    OBJGPU *pGpu,
+    KernelNvlink *pKernelNvlink
+)
+{
+    return (pKernelNvlink == NULL) ? 0 : portAtomicExOrU64(&pKernelNvlink->nvlinkBwModeEpoch, 0);
+}
+
+void
+knvlinkSetBWModeEpoch_IMPL
+(
+    OBJGPU *pGpu,
+    KernelNvlink *pKernelNvlink,
+    NvU64 bwModeEpoch
+)
+{
+    portAtomicExSetU64(&pKernelNvlink->nvlinkBwModeEpoch, bwModeEpoch);
+}
+
 void
 knvlinkUtoa(NvU8 *str, NvU64 length, NvU64 val)
 {
